@@ -1,7 +1,7 @@
 package CPAN::Reporter;
 use strict;
 
-$CPAN::Reporter::VERSION = '1.00'; 
+$CPAN::Reporter::VERSION = '1.07_01'; 
 
 use Config;
 use CPAN ();
@@ -11,13 +11,14 @@ use File::Find ();
 use File::HomeDir ();
 use File::Path qw/mkpath rmtree/;
 use File::Spec ();
-use File::Temp ();
+use File::Temp 0.16 ();
 use IO::File ();
 use Probe::Perl ();
 use Tee qw/tee/;
 use Test::Reporter ();
 use CPAN::Reporter::Config ();
 use CPAN::Reporter::History ();
+use CPAN::Reporter::PrereqCheck ();
 
 use constant MAX_OUTPUT_LENGTH => 50_000;
 
@@ -33,9 +34,16 @@ sub grade_make {
     my @args = @_;
     my $result = _init_result( 'make', @args ); 
     _compute_make_grade($result);
-    _print_grade_msg($result->{is_make} ? $Config{make} : 'Build' , $result);
-    if( $result->{grade} ne 'pass' ) {
-        _dispatch_report( $result );
+    if ( $result->{grade} eq 'discard' ) {
+        $CPAN::Frontend->mywarn( 
+            "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
+            $result->{prereq_pm}, "\n",
+            "Test results for $result->{dist_name} will be discarded"
+        );
+    }
+    else {
+        _print_grade_msg($result->{make_cmd}, $result);
+        if ( $result->{grade} ne 'pass' ) { _dispatch_report( $result ) }
     }
     return $result->{success};
 }
@@ -44,9 +52,16 @@ sub grade_PL {
     my @args = @_;
     my $result = _init_result( 'PL', @args ); ## no critic
     _compute_PL_grade($result);
-    _print_grade_msg($result->{PL_file} , $result);
-    if( $result->{grade} ne 'pass' ) {
-        _dispatch_report( $result );
+    if ( $result->{grade} eq 'discard' ) {
+        $CPAN::Frontend->mywarn( 
+            "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
+            $result->{prereq_pm}, "\n",
+            "Test results for $result->{dist_name} will be discarded"
+        );
+    }
+    else {
+        _print_grade_msg($result->{PL_file} , $result);
+        if ( $result->{grade} ne 'pass' ) { _dispatch_report( $result ) }
     }
     return $result->{success};
 }
@@ -57,7 +72,7 @@ sub grade_test {
     _compute_test_grade($result);
     if ( $result->{grade} eq 'discard' ) {
         $CPAN::Frontend->mywarn( 
-            "\nCPAN::Reporter: Test results were not valid, $result->{grade_msg}.\n\n",
+            "\nCPAN::Reporter: test results were not valid, $result->{grade_msg}.\n\n",
             $result->{prereq_pm}, "\n",
             "Test results for $result->{dist_name} will be discarded"
         );
@@ -74,13 +89,15 @@ sub record_command {
 
     my ($cmd, $redirect) = _split_redirect($command);
 
-    my $temp_out = File::Temp->new
-        or die "Could not create a temporary file for output: $!";
+    my $temp_out = File::Temp->new( 
+        TEMPLATE => 'CPAN-Reporter-TO-XXXXXXXX', DIR => File::Spec->tmpdir()
+    ) or die "Could not create a temporary file for output: $!";
 
     # Teeing a command loses its exit value so we must wrap the command 
     # and print the exit code so we can read it off of output
-    my $cmdwrapper = File::Temp->new
-        or die "Could not create a wrapper for $cmd\: $!";
+    my $cmdwrapper = File::Temp->new( 
+        TEMPLATE => 'CPAN-Reporter-CW-XXXXXXXX', DIR => File::Spec->tmpdir() 
+    ) or die "Could not create a wrapper for $cmd\: $!";
 
     my $wrap_code;
     if ( $timeout ) {
@@ -90,10 +107,11 @@ sub record_command {
     }
     # if no timeout or timeout wrap code wasn't available
     if ( ! $wrap_code ) {
+        my $safecmd = quotemeta($cmd);
         $wrap_code = << "HERE";
-my \$rc = system('$cmd');
+my \$rc = system("$safecmd");
 my \$ec = \$rc == -1 ? -1 : \$?;
-print '($cmd exited with ', \$ec, ")\\n";
+print "($safecmd exited with \$ec)\\n";
 HERE
     }
 
@@ -109,14 +127,14 @@ HERE
     my $temp_out2 = IO::File->new($temp_out->filename, "<");
     if ( !$temp_out2 ) {
         $CPAN::Frontend->mywarn( 
-            "CPAN::Reporter couldn't read command results for '$cmd'\n" 
+            "CPAN::Reporter: couldn't read command results for '$cmd'\n" 
         );
         return;
     }
     my @cmd_output = <$temp_out2>;
     if ( ! @cmd_output ) {
         $CPAN::Frontend->mywarn( 
-            "CPAN::Reporter didn't capture command results for '$cmd'\n"
+            "CPAN::Reporter: didn't capture command results for '$cmd'\n"
         );
         return;
     }
@@ -129,7 +147,7 @@ HERE
     }
     if ( ! defined $exit_value || $exit_value == -1 ) {
         $CPAN::Frontend->mywarn( 
-            "CPAN::Reporter couldn't execute '$cmd'\n"
+            "CPAN::Reporter: couldn't execute '$cmd'\n"
         );
         return;
     }
@@ -142,7 +160,7 @@ sub test {
     my ($output, $exit_value) = record_command( $system_command );
     unless ( defined $output && defined $exit_value ) {
         $CPAN::Frontend->mywarn(
-            "CPAN::Reporter had errors capturing output. Tests abandoned"
+            "CPAN::Reporter: had errors capturing output. Tests abandoned"
         );
         return;
     }
@@ -154,7 +172,7 @@ sub test {
 #--------------------------------------------------------------------------#
 
 #--------------------------------------------------------------------------#
-# _compute_PL_grade
+# _compute_make_grade
 #--------------------------------------------------------------------------#
 
 sub _compute_make_grade {
@@ -168,34 +186,34 @@ sub _compute_make_grade {
         $result->{grade} = "pass";
         $result->{grade_msg} = "No errors"
     }
-    $result->{success} = $result->{grade} eq "pass" ? 1 : 0;
+
+    _downgrade_known_causes( $result );
+    
+    $result->{success} =  $result->{grade} eq 'pass'
+                       || $result->{grade} eq 'unknown';
     return;
 }
 
+#--------------------------------------------------------------------------#
+# _compute_PL_grade
+#--------------------------------------------------------------------------#
+
 sub _compute_PL_grade {
     my $result = shift;
     my ($grade,$msg);
     if ( $result->{exit_value} ) {
-        if (grep { /Perl .*? required.*?--this is only .*?/ } 
-                    @{$result->{output}}) {
-            $result->{grade} = "na";
-            $result->{grade_msg} = "Perl version too low";
-        }
-        elsif ( grep { /OS Unsupported|No support for OS/i }  
-                    @{$result->{output}}) {
-            $result->{grade} = "na";
-            $result->{grade_msg} = "This platform is not supported"
-        }
-        else {
-            $result->{grade} = "fail";
-            $result->{grade_msg} = "Stopped with an error"
-        }
+        $result->{grade} = "fail";
+        $result->{grade_msg} = "Stopped with an error"
     }
     else {
         $result->{grade} = "pass";
         $result->{grade_msg} = "No errors"
     }
-    $result->{success} = $result->{grade} eq "pass" ? 1 : 0;
+
+    _downgrade_known_causes( $result );
+    
+    $result->{success} =  $result->{grade} eq 'pass'
+                       || $result->{grade} eq 'unknown';
     return;
 }
 
@@ -224,9 +242,6 @@ sub _compute_test_grade {
     my ($grade,$msg);
     my $output = $result->{output};
 
-    # we need to find prerequisites and toolchain earlier than usual
-    _expand_result( $result );
-
     # In some cases, get a result straight from the exit code 
     if ( $result->{is_make} && ( -f "test.pl" || _has_recursive_make() ) ) {
         if ( $result->{exit_value} ) {
@@ -241,6 +256,7 @@ sub _compute_test_grade {
     # Otherwise, get a result from Test::Harness output
     else {
         # figure out the right harness parser
+        _expand_result( $result ); 
         my $harness_version = $result->{toolchain}{'Test::Harness'}{have};
         my $harness_parser = CPAN::Version->vgt($harness_version, '2.99_01')
                     ? \&_parse_tap_harness
@@ -251,7 +267,7 @@ sub _compute_test_grade {
                 $grade = 'na';
                 $msg = 'This platform is not supported';
             }
-            elsif ( $output->[$i] =~ m{^.?No tests defined}ms ) { # from EU::MM
+            elsif ( $output->[$i] =~ m{^.?No tests defined}ms ) { # from M::B
                 $grade = 'unknown';
                 $msg = 'No tests provided';
             }
@@ -268,37 +284,11 @@ sub _compute_test_grade {
         }
     }
 
-    # Downgrade failure/unknown grade if we can determine a cause
-    # If platform not supported => 'na'
-    # If perl version is too low => 'na'
-    # If stated prereqs missing => 'discard'
-
-    if ( $grade eq 'fail' || $grade eq 'unknown' ) {
-        # check again for unsupported OS in case we took 'fail' from exit value
-        if ( grep { /No support for OS|OS unsupported/ims } @{$output} ) {
-            $grade = 'na';
-            $msg = 'This platform is not supported';
-        }
-        # check for perl version prerequisite or outright failure
-        if ( $result->{prereq_pm} =~ m{^\s+!\s+perl\s}ims 
-          || grep { /Perl .*? required.*?--this is only/ms } @{$output}
-        ) {
-            $grade = 'na';
-            $msg = 'Perl version too low';
-        }
-        # check the prereq report for missing or failure flag '!'
-        elsif ( $result->{prereq_pm} =~ m{n/a}ims ) {
-            $grade = 'discard';
-            $msg = 'Prerequisite missing';
-        }
-        elsif ( $result->{prereq_pm} =~ m{^\s+!}ims ) {
-            $grade = 'discard';
-            $msg = 'Prerequisite version too low';
-        }
-    }
-
     $result->{grade} = $grade;
     $result->{grade_msg} = $msg;
+
+    _downgrade_known_causes( $result );
+
     $result->{success} =  $result->{grade} eq 'pass'
                        || $result->{grade} eq 'unknown';
     return;
@@ -315,7 +305,7 @@ sub _dispatch_report {
     my $phase = $result->{phase};
 
     $CPAN::Frontend->myprint(
-        "Preparing a CPAN Testers report for $result->{dist_name}\n"
+        "CPAN::Reporter: preparing a CPAN Testers report for $result->{dist_name}\n"
     );
 
     # Get configuration options
@@ -326,8 +316,8 @@ sub _dispatch_report {
     if ( ! $config->{email_from} ) {
         $CPAN::Frontend->mywarn( << "EMAIL_REQUIRED");
         
-CPAN::Reporter requires an email-address in the config file.  
-Test report will not be sent. See documentation for configuration details.
+CPAN::Reporter: required 'email_from' option missing an email address, so
+test report will not be sent. See documentation for configuration details.
 
 EMAIL_REQUIRED
         return;
@@ -343,9 +333,9 @@ EMAIL_REQUIRED
     if ( ! grep { length } @format_checks ) {
         $CPAN::Frontend->mywarn( << "END_BAD_DISTNAME");
         
-The distribution name '$result->{dist_basename}' does not appear to be 
-formatted according to CPAN tester guidelines. Perhaps it is not a normal
-CPAN distribution.
+CPAN::Reporter: the distribution name '$result->{dist_basename}' does not 
+appear to be packaged according to CPAN tester guidelines. Perhaps it is 
+not a normal CPAN distribution.
 
 Test report will not be sent.
 
@@ -357,6 +347,28 @@ END_BAD_DISTNAME
     # Gather 'expensive' data for the report
     _expand_result( $result);
 
+    # Skip if distribution name matches the send_skipfile
+    if ( $config->{send_skipfile} && -r $config->{send_skipfile} ) {
+        my $send_skipfile = IO::File->new( $config->{send_skipfile}, "r" );
+        my $dist_id = $result->{dist}->pretty_id;
+        while ( my $pattern = <$send_skipfile> ) {
+            chomp($pattern);
+            # ignore comments
+            next if substr($pattern,0,1) eq '#';
+            # if it doesn't match, continue with next pattern
+            next if $dist_id !~ /$pattern/;
+            # if it matches, warn and return
+            $CPAN::Frontend->mywarn( << "END_SKIP_DIST" );
+CPAN::Reporter: '$dist_id' matched against the send_skipfile.  
+
+Test report will not be sent.
+
+END_SKIP_DIST
+
+            return;
+        }
+    }
+
     # Setup the test report
     my $tr = Test::Reporter->new;
     $tr->grade( $result->{grade} );
@@ -368,7 +380,7 @@ END_BAD_DISTNAME
         if ( _prompt( $config, "send_duplicates", $tr->grade) =~ /^n/ ) {
             $CPAN::Frontend->mywarn(<< "DUPLICATE_REPORT");
 
-It seems that "@{[$tr->subject]}"
+CPAN::Reporter: it seems that "@{[$tr->subject]}"
 during the $phase phase is a duplicate of a previous report you 
 sent to CPAN Testers.
 
@@ -385,8 +397,8 @@ DUPLICATE_REPORT
     my $transport = $config->{transport} || 'Net::SMTP';
     if (length $transport && ( $transport !~ /\ANet::SMTP|Mail::Send\z/ )) {
         $CPAN::Frontend->mywarn(
-            "CPAN::Reporter doesn't recognize '$config->{transport}' as a valid transport.\n" .
-            "Falling back to Net::SMTP\n"
+            "CPAN::Reporter: '$config->{transport}' is not a valid transport option." .
+            " Falling back to Net::SMTP\n"
         );
         $transport = 'Net::SMTP';
     }
@@ -403,14 +415,9 @@ DUPLICATE_REPORT
     # Populate the test report
     $tr->comments( _report_text( $result ) );
     $tr->via( 'CPAN::Reporter ' . $CPAN::Reporter::VERSION );
-    my @cc;
+    my @cc = _should_copy_author( $result, $config );
 
-    # User prompts for action
-    if ( _prompt( $config, "cc_author", $tr->grade) =~ /^y/ ) {
-        # CC only if we have an author_id
-        push @cc, "$result->{author_id}\@cpan.org" if $result->{author_id};
-    }
-    
+    # prompt for editing report
     if ( _prompt( $config, "edit_report", $tr->grade ) =~ /^y/ ) {
         my $editor = $config->{editor};
         local $ENV{VISUAL} = $editor if $editor; ## no critic
@@ -422,23 +429,85 @@ DUPLICATE_REPORT
                     ? "send_$phase\_report"
                     : "send_report" ;
     if ( _prompt( $config, $send_config, $tr->grade ) =~ /^y/ ) {
-        $CPAN::Frontend->myprint( "Sending test report with '" . $tr->grade . 
+        $CPAN::Frontend->myprint( "CPAN::Reporter: sending test report with '" . $tr->grade . 
               "' to " . join(q{, }, $tr->address, @cc) . "\n");
         if ( $tr->send( @cc ) ) {
             CPAN::Reporter::History::_record_history( $result ) 
                 if not $is_duplicate;
         }
         else {
-            $CPAN::Frontend->mywarn( $tr->errstr. "\n");
+            $CPAN::Frontend->mywarn( "CPAN::Reporter: " . $tr->errstr . "\n");
         }
     }
     else {
-        $CPAN::Frontend->myprint("Test report will not be sent\n");
+        $CPAN::Frontend->myprint("CPAN::Reporter: test report will not be sent\n");
     }
 
     return;
 }
 
+#--------------------------------------------------------------------------#
+# _downgrade_known_causes
+# Downgrade failure/unknown grade if we can determine a cause
+# If platform not supported => 'na'
+# If perl version is too low => 'na'
+# If stated prereqs missing => 'discard'
+#--------------------------------------------------------------------------#
+
+sub _downgrade_known_causes {
+    my ($result) = @_;
+    my ($grade, $output) = ( $result->{grade}, $result->{output} );
+    my $msg = $result->{grade_msg} || q{};
+
+    # shortcut unless fail/unknown
+    return if $grade eq 'pass' || $grade eq 'na';
+
+    # get prereqs
+    _expand_result( $result ); 
+
+    # look for perl version error messages from various programs
+    # "Error evaling..." type errors happen on Perl < 5.006 when modules
+    # define their version with "our $VERSION = ..."
+    my $version_error;
+    for my $line ( @$output ) {
+        if( $line =~ /Perl .*? required.*?--this is only/ims ||
+            $line =~ /ERROR: perl: Version .*? is installed, but we need version/ims ||
+            $line =~ /ERROR: perl \(.*?\) is installed, but we need version/ims ||
+            $line =~ /Error evaling version line 'BEGIN/ims ||
+            $line =~ /Could not eval '/ims
+        ) {
+            $version_error++;
+            last;
+        }
+    }
+
+    # check for explicit version error or just a perl version prerequisite
+    if ( $version_error || $result->{prereq_pm} =~ m{^\s+!\s+perl\s}ims ) {
+        $grade = 'na';
+        $msg = 'Perl version too low';
+    }
+    # check again for unsupported OS in case we took 'fail' from exit value
+    elsif ( grep { /No support for OS|OS unsupported/ims } @{$output} ) {
+        $grade = 'na';
+        $msg = 'This platform is not supported';
+    }
+    # check the prereq report for missing or failure flag '!'
+    elsif ( $result->{prereq_pm} =~ m{n/a}ims ) {
+        $grade = 'discard';
+        $msg = 'Prerequisite missing';
+    }
+    elsif ( $result->{prereq_pm} =~ m{^\s+!}ims ) {
+        $grade = 'discard';
+        $msg = 'Prerequisite version too low';
+    }
+
+    # store results
+    $result->{grade} = $grade;
+    $result->{grade_msg} = $msg;
+
+    return;
+}
+
 #--------------------------------------------------------------------------#
 # _expand_result - add expensive information like prerequisites and
 # toolchain that should only be generated if a report will actually
@@ -510,26 +579,6 @@ sub _format_distname {
     return $basename;
 }
 
-
-#--------------------------------------------------------------------------#
-# _has_tests
-#--------------------------------------------------------------------------#
-
-sub _has_tests {
-    return 1 if -f 'test.pl';
-    if ( -d 't' ) {
-        local *TESTDIR;
-        opendir TESTDIR, 't';
-        while ( my $f = readdir TESTDIR ) {
-            if ( $f =~ m{\.t$} ) {
-                close TESTDIR;
-                return 1;
-            }
-        }
-    }
-    return 0;
-}
-
 #--------------------------------------------------------------------------#
 # _has_recursive_make
 #
@@ -576,6 +625,7 @@ sub _init_result {
 
     # Used in messages to user
     $result->{PL_file} = $result->{is_make} ? "Makefile.PL" : "Build.PL";
+    $result->{make_cmd} = $result->{is_make} ? $Config{make} : "Build";
 
     # CPAN might fail to find an author object for some strange dists
     my $author = $dist->author;
@@ -758,24 +808,22 @@ sub _print_grade_msg {
 #--------------------------------------------------------------------------#
 
 sub _prompt {
-    my ($config, $option, $grade) = @_;
+    my ($config, $option, $grade, $extra) = @_;
+    $extra ||= q{};
+
     my %spec = CPAN::Reporter::Config::_config_spec();
 
     my $dispatch = CPAN::Reporter::Config::_validate_grade_action_pair(
         $option, join(q{ }, "default:no", $config->{$option} || '')
     );
     my $action = $dispatch->{$grade} || $dispatch->{default};
-
+    my $intro = $spec{$option}{prompt} . $extra . " (yes/no)";
     my $prompt;
     if     ( $action =~ m{^ask/yes}i ) { 
-        $prompt = CPAN::Shell::colorable_makemaker_prompt( 
-            $spec{$option}{prompt} . " (yes/no)", "yes" 
-        );
+        $prompt = CPAN::Shell::colorable_makemaker_prompt( $intro, "yes" );
     }
     elsif  ( $action =~ m{^ask(/no)?}i ) {
-        $prompt = CPAN::Shell::colorable_makemaker_prompt( 
-            $spec{$option}{prompt} . " (yes/no)", "no" 
-        );
+        $prompt = CPAN::Shell::colorable_makemaker_prompt( $intro, "no" );
     }
     else { 
         $prompt = $action;
@@ -796,8 +844,10 @@ HERE
     'fail' => <<'HERE',
 Thank you for uploading your work to CPAN.  However, it appears that
 there were some problems with your distribution.  If these results are 
-not what you expect, please consult "Notes for CPAN Authors" on 
-the CPAN Testers Wiki: http://cpantest.grango.org
+not what you expect or if you would like to learn how to avoid FAIL 
+reports for missing dependencies, unsupported operating systems, etc.,
+please consult "Notes for CPAN Authors" on the CPAN Testers Wiki: 
+http://cpantest.grango.org
 HERE
 
     'unknown' => <<'HERE',
@@ -886,6 +936,50 @@ ENDREPORT
     return $output;
 }
 
+
+#--------------------------------------------------------------------------#
+# _should_copy_author
+#--------------------------------------------------------------------------#
+
+sub _should_copy_author {
+    my ($result, $config) = @_;
+
+    # User prompts for action
+    my $author_email = $result->{author_id} 
+                     ? "$result->{author_id}\@cpan.org"
+                     : q{};
+    if ( ! $author_email ) {
+        $CPAN::Frontend->mywarn( "CPAN::Reporter: couldn't determine author_id and won't cc author.\n");
+        return;
+    }
+
+    # Skip if distribution name matches the cc_skipfile
+    if ( $config->{cc_skipfile} && -r $config->{cc_skipfile} ) {
+        my $cc_skipfile = IO::File->new( $config->{cc_skipfile}, "r" );
+        my $dist_id = $result->{dist}->pretty_id;
+        while ( my $pattern = <$cc_skipfile> ) {
+            chomp($pattern);
+            # ignore comments
+            next if substr($pattern,0,1) eq '#';
+            # if it doesn't match, continue with next pattern
+            next if $dist_id !~ /$pattern/;
+            # if it matches, warn and return
+            $CPAN::Frontend->mywarn( << "END_SKIP_DIST" );
+CPAN::Reporter: '$dist_id' matched against the cc_skipfile.  Won't copy author.
+END_SKIP_DIST
+            return;
+        }
+    }
+
+    # Finally, prompt user if necessary
+    if ( _prompt( $config, "cc_author", $result->{grade}, "($author_email)?") =~ /^y/ ) {
+        return $author_email;
+    }
+    else {
+        return;
+    }
+}
+
 #--------------------------------------------------------------------------#
 # _special_vars_report
 #--------------------------------------------------------------------------#
@@ -929,6 +1023,9 @@ sub _split_redirect {
 sub _timeout_wrapper {
     my ($cmd, $timeout) = @_;
     
+    # protect shell quotes
+    $cmd = quotemeta($cmd);
+
     my $wrapper = sprintf << 'HERE', $timeout, $cmd, $cmd;
 use strict;
 my ($pid, $exitcode);
@@ -942,7 +1039,7 @@ eval {
         my $wstat = waitpid $pid, 0;
         $exitcode = $wstat == -1 ? -1 : $?;
     } else {    #child
-        exec '%s';
+        exec "%s";
     }
 };
 alarm 0;
@@ -954,7 +1051,7 @@ if ($pid && $@ =~ /Timeout/){
 elsif ($@) {
     die $@;
 }
-print '(%s exited with ', $exitcode, ")\n";
+print "(%s exited with $exitcode)\n";
 HERE
     return $wrapper;
 }
@@ -969,7 +1066,7 @@ sub _timeout_wrapper_win32 {
     eval "use Win32::Process 0.10 ();";
     if ($@) {
         $CPAN::Frontend->mywarn( << 'HERE' );
-CPAN::Reporter needs Win32::Process 0.10 for inactivity_timeout support.
+CPAN::Reporter: you need Win32::Process 0.10 for inactivity_timeout support.
 Continuing without timeout...
 HERE
         return;
@@ -982,7 +1079,7 @@ HERE
                      split /$Config{path_sep}/, $ENV{PATH};
         if (! $path) {
             $CPAN::Frontend->mywarn( << "HERE" );
-CPAN::Reporter can't locate $exe in the PATH. 
+CPAN::Reporter: can't locate $exe in the PATH. 
 Continuing without timeout...
 HERE
             return;
@@ -990,25 +1087,28 @@ HERE
         $program = File::Spec->catfile($path,$exe);
     }
 
+    # protect shell quotes and other things
+    $_ = quotemeta($_) for ($program, $cmd);
+
     my $wrapper = sprintf << 'HERE', $program, $cmd, $cmd, $timeout, $cmd;
 use strict;
 use Win32::Process qw/STILL_ACTIVE NORMAL_PRIORITY_CLASS/;
 my ($process,$exitcode);
 Win32::Process::Create(
     $process,
-    '%s',
-    '%s',
+    "%s",
+    "%s",
     0,
     NORMAL_PRIORITY_CLASS,
     "."
-) or die 'Could not spawn %s: ' . "$^E\n";
+) or die "Could not spawn %s: $^E\n";
 $process->Wait(%s * 1000);
 $process->GetExitCode($exitcode);
 if ($exitcode == STILL_ACTIVE) {
     $process->Kill(9);
     $exitcode = 9;
 }
-print '(%s exited with ', $exitcode, ")\n";
+print "(%s exited with $exitcode)\n";
 HERE
     return $wrapper;
 }
@@ -1078,89 +1178,7 @@ sub _toolchain_report {
 #
 #--------------------------------------------------------------------------#
 
-my $version_finder = File::Temp->new
-    or die "Could not create temporary support program for versions: $!";
-$version_finder->print( << 'END' );
-use strict;
-use ExtUtils::MakeMaker;
-use CPAN::Version;
-
-# read module and prereq string from STDIN
-while ( <STDIN> ) {
-    m/^(\S+)\s+([^\n]*)/;
-    my ($mod, $need) = ($1, $2);
-    die "Couldn't read module for '$_'" unless $mod;
-    $need = 0 if not defined $need;
-
-    # get installed version from file with EU::MM
-    my($have, $inst_file, $dir, @packpath);
-    if ( $mod eq "perl" ) { 
-        $have = $];
-    }
-    else {
-        @packpath = split /::/, $mod;
-        $packpath[-1] .= ".pm";
-        if (@packpath == 1 && $packpath[0] eq "readline.pm") {
-            unshift @packpath, "Term", "ReadLine"; # historical reasons
-        }
-        foreach $dir (@INC) {
-            my $pmfile = File::Spec->catfile($dir,@packpath);
-            if (-f $pmfile){
-                $inst_file = $pmfile;
-            }
-        }
-        
-        # get version from file or else report missing
-        if ( defined $inst_file ) {
-            $have = MM->parse_version($inst_file);
-            $have = "0" if ! defined $have || $have eq 'undef';
-        }
-        else {
-            print "$mod 0 n/a\n";
-            next;
-        }
-    }
-
-    # complex requirements are comma separated
-    my ( @requirements ) = split /\s*,\s*/, $need;
-
-    my $passes = 0;
-    RQ: 
-    for my $rq (@requirements) {
-        if ($rq =~ s|>=\s*||) {
-            # no-op -- just trimmed string
-        } elsif ($rq =~ s|>\s*||) {
-            if (CPAN::Version->vgt($have,$rq)){
-                $passes++;
-            }
-            next RQ;
-        } elsif ($rq =~ s|!=\s*||) {
-            if (CPAN::Version->vcmp($have,$rq)) { 
-                $passes++; # didn't match
-            }
-            next RQ;
-        } elsif ($rq =~ s|<=\s*||) {
-            if (! CPAN::Version->vgt($have,$rq)){
-                $passes++;
-            }
-            next RQ;
-        } elsif ($rq =~ s|<\s*||) {
-            if (CPAN::Version->vlt($have,$rq)){
-                $passes++;
-            }
-            next RQ;
-        }
-        # if made it here, then it's a normal >= comparison
-        if (! CPAN::Version->vlt($have, $rq)){
-            $passes++;
-        }
-    }
-    my $ok = $passes == @requirements ? 1 : 0;
-    print "$mod $ok $have\n"
-}
-END
-
-$version_finder->close;
+my $version_finder = $INC{'CPAN/Reporter/PrereqCheck.pm'};
 
 sub _version_finder {
     my %prereqs = @_;
@@ -1168,8 +1186,9 @@ sub _version_finder {
     my $perl = Probe::Perl->find_perl_interpreter();
     my @prereq_results;
     
-    my $prereq_input = File::Temp->new
-        or die "Could not create temporary input for prereq analysis: $!";
+    my $prereq_input = File::Temp->new( 
+        TEMPLATE => 'CPAN-Reporter-PI-XXXXXXXX', DIR => File::Spec->tmpdir() 
+    ) or die "Could not create temporary input for prereq analysis: $!";
     $prereq_input->print( map { "$_ $prereqs{$_}\n" } keys %prereqs );
     $prereq_input->close;
 
@@ -1362,11 +1381,6 @@ you may not use this file except in compliance with the License.
 You may obtain a copy of the License at 
 [http://www.apache.org/licenses/LICENSE-2.0]
 
-Files produced as output though the use of this software, including
-generated copies of boilerplate templates provided with this software,
-shall not be considered Derivative Works, but shall be considered the
-original work of the Licensor.
-
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
