@@ -1,6 +1,8 @@
 package CPAN::Reporter::Config;
-$VERSION = '1.00'; ## no critic
 use strict; 
+use vars qw/$VERSION/;
+$VERSION = '1.07_01'; 
+
 use Config::Tiny ();
 use File::HomeDir (); 
 use File::Path (qw/mkpath/);
@@ -19,7 +21,8 @@ if ( $^O eq 'darwin' ) {
     my $new = File::Spec->catdir(File::HomeDir->my_home,".cpanreporter");
     if ( ( -d $old ) && (! -d $new ) ) {
         $CPAN::Frontend->mywarn( << "HERE");
-Since CPAN::Reporter 0.28_51, the Mac OSX config directory has changed. 
+CPAN::Reporter: since CPAN::Reporter 0.28_51, the Mac OSX config directory 
+has changed. 
 
   Old: $old
   New: $new  
@@ -52,7 +55,7 @@ sub _configure {
     mkpath $config_dir if ! -d $config_dir;
     if ( ! -d $config_dir ) {
         $CPAN::Frontend->myprint(
-            "\nCouldn't create configuration directory '$config_dir': $!"
+            "\nCPAN::Reporter: couldn't create configuration directory '$config_dir': $!"
         );
         return;
     }
@@ -66,24 +69,24 @@ sub _configure {
     # read or create
     if ( -f $config_file ) {
         $CPAN::Frontend->myprint(
-            "\nFound your CPAN::Reporter config file at:\n$config_file\n"
+            "\nCPAN::Reporter: found your CPAN::Reporter config file at:\n$config_file\n"
         );
         $config = _open_config_file();
         # if we can't read it, bail out
         if ( ! $config ) {
             $CPAN::Frontend->mywarn("\n
-                CPAN::Reporter configuration will not be changed\n");
+                CPAN::Reporter: configuration will not be changed\n");
             return;
         }
         # clone what's in the config file
         $existing_options = { %{$config->{_}} } if $config;
         $CPAN::Frontend->myprint(
-            "\nUpdating your CPAN::Reporter configuration settings:\n"
+            "\nCPAN::Reporter: Updating your CPAN::Reporter configuration settings:\n"
         );
     }
     else {
         $CPAN::Frontend->myprint(
-            "\nNo CPAN::Reporter config file found; creating a new one.\n"
+            "\nCPAN::Reporter: no config file found; creating a new one.\n"
         );
         $config = Config::Tiny->new();
     }
@@ -147,14 +150,14 @@ sub _configure {
     }
 
     $CPAN::Frontend->myprint( 
-        "\nWriting CPAN::Reporter config file to '$config_file'.\n"
+        "\nCPAN::Reporter: writing config file to '$config_file'.\n"
     );
     if ( $config->write( $config_file ) ) {
         return $config->{_};
     }
     else {
-        $CPAN::Frontend->mywarn( "\nError writing config file to '$config_file':" . 
-             Config::Tiny->errstr(). "\n");
+        $CPAN::Frontend->mywarn( "\nCPAN::Reporter: error writing config file to '$config_file':\n" 
+            .  Config::Tiny->errstr(). "\n");
         return;
     }
 }
@@ -227,7 +230,7 @@ HERE
     },
     send_report => {
         default => 'default:ask/yes pass/na:yes',
-        prompt => "Do you want to send the test report?",
+        prompt => "Do you want to send the report?",
         validate => \&_validate_grade_action_pair,
         info => <<'HERE',
 By default, CPAN::Reporter will prompt you for confirmation that
@@ -239,7 +242,7 @@ HERE
     },
     cc_author => {
         default => 'default:yes pass/na:no',
-        prompt => "Do you want to CC the the module author?",
+        prompt => "Do you want to CC the module author ", # (author@cpan.org) added dynamically
         validate => \&_validate_grade_action_pair,
         info => <<'HERE',
 If you would like, CPAN::Reporter will copy the module author with
@@ -259,13 +262,29 @@ all, regardless of the value of the "send_report" option.  This option takes
 HERE
     },
     send_PL_report => {
+        prompt => "Do you want to send the PL report?",
         default => undef, 
+        validate => \&_validate_grade_action_pair,
     },
     send_make_report => {
+        prompt => "Do you want to send the make/Build report?",
         default => undef,
+        validate => \&_validate_grade_action_pair,
     },
     send_test_report => {
+        prompt => "Do you want to send the test report?",
         default => undef,
+        validate => \&_validate_grade_action_pair,
+    },
+    send_skipfile => {
+        prompt => "What file has patterns for things that shouldn't be reported?",
+        default => undef,
+        validate => \&_validate_skipfile,
+    },
+    cc_skipfile => {
+        prompt => "What file has patterns for things that shouldn't CC to authors?",
+        default => undef,
+        validate => \&_validate_skipfile,
     },
     email_to => {
         default => undef,
@@ -316,7 +335,7 @@ sub _get_config_options {
             if  (   $spec{$option}{validate} &&
                     ! $spec{$option}{validate}->($option, $val)
                 ) {
-                    $CPAN::Frontend->mywarn( "\nInvalid option '$val' in '$option'. Using default instead.\n\n" );
+                    $CPAN::Frontend->mywarn( "\nCPAN::Reporter: invalid option '$val' in '$option'. Using default instead.\n\n" );
                     $active{$option} = $spec{$option}{default};
                     next OPTION;
             }
@@ -330,6 +349,7 @@ sub _get_config_options {
     return \%active;
 }
 
+
 #--------------------------------------------------------------------------#
 # _grade_action_prompt -- describes grade action pairs
 #--------------------------------------------------------------------------#
@@ -380,8 +400,8 @@ sub _is_valid_grade {
 sub _open_config_file {
     my $config_file = _get_config_file();
     my $config = Config::Tiny->read( $config_file )
-        or $CPAN::Frontend->mywarn("Couldn't read CPAN::Reporter configuration file " .
-                "'$config_file': " . Config::Tiny->errstr() . "\n");
+        or $CPAN::Frontend->mywarn("CPAN::Reporter: couldn't read configuration file " .
+                "'$config_file': \n" . Config::Tiny->errstr() . "\n");
     return $config; 
 }
 
@@ -422,7 +442,7 @@ sub _validate_grade_action_pair {
         else {
             # something weird, so warn and skip
             $CPAN::Frontend->mywarn( 
-                "\nIgnoring invalid grade:action '$grade_action' for '$name'.\n\n" 
+                "\nCPAN::Reporter: ignoring invalid grade:action '$grade_action' for '$name'.\n\n" 
             );
             next PAIR;
         }
@@ -432,7 +452,7 @@ sub _validate_grade_action_pair {
         for my $g ( keys %grades ) { 
             if ( ! _is_valid_grade($g) ) {
                 $CPAN::Frontend->mywarn( 
-                    "\nIgnoring invalid grade '$g' in '$grade_action' for '$name'.\n\n" 
+                    "\nCPAN::Reporter: ignoring invalid grade '$g' in '$grade_action' for '$name'.\n\n" 
                 );
                 delete $grades{$g};
             }
@@ -441,7 +461,7 @@ sub _validate_grade_action_pair {
         # check action
         if ( ! _is_valid_action($action) ) {
             $CPAN::Frontend->mywarn( 
-                "\nIgnoring invalid action '$action' in '$grade_action' for '$name'.\n\n" 
+                "\nCPAN::Reporter: ignoring invalid action '$action' in '$grade_action' for '$name'.\n\n" 
             );
             next PAIR;
         }
@@ -453,6 +473,14 @@ sub _validate_grade_action_pair {
     return scalar(keys %ga_map) ? \%ga_map : undef;
 }
 
+sub _validate_skipfile {
+    my ($name, $option) = @_;
+    return unless $option;
+    my $skipfile = File::Spec->file_name_is_absolute( $option )
+                 ? $option : File::Spec->catfile( _get_config_dir(), $option );
+    return -r $skipfile ? $skipfile : undef;
+}
+
 1;
 __END__
 
@@ -593,6 +621,11 @@ certain situations.
 
 * {cc_author = <grade:action> ...} -- should module authors should be sent a copy of 
 the test report at their {author@cpan.org} address? (default:yes pass/na:no)
+* {cc_skipfile = <skipfile>} -- filename containing regular expressions (one
+per line) to match against the distribution ID (e.g. 
+'AUTHOR/Dist-Name-0.01.tar.gz'); the author will not be copied if a match is 
+found regardless of cc_author; non-absolute filename must be in the .cpanreporter 
+config directory
 * {editor = <editor>} -- editor to use to edit the test report; if not set,
 Test::Reporter will use environment variables {VISUAL}, {EDITOR} or {EDIT}
 (in that order) to find an editor 
@@ -604,6 +637,8 @@ reports be sent, regardless of {send_report}? (default:no)
 {send_report} during the make phase
 * {send_test_report = <grade:action> ...} -- if defined, used in place of 
 {send_report} during the test phase
+* {send_skipfile = <skipfile>} -- like {cc_skipfile} but no report will be 
+sent at all if a match is found
 * {transport = <transport>} -- if defined, passed to the {transport()} 
 method of [Test::Reporter].  Valid options are 'Net::SMTP' or 
 'Mail::Send'.  (CPAN::Reporter uses Net::SMTP for this by default.)
@@ -637,11 +672,6 @@ you may not use this file except in compliance with the License.
 You may obtain a copy of the License at 
 [http://www.apache.org/licenses/LICENSE-2.0]
 
-Files produced as output though the use of this software, including
-generated copies of boilerplate templates provided with this software,
-shall not be considered Derivative Works, but shall be considered the
-original work of the Licensor.
-
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
